from starlette.config import Config

config = Config()


class Settings:
    """Settings for the module"""

    TRANSACTION_MIDDLEWARE_LOG_LEVEL: str = config(
        "TRANSACTION_MIDDLEWARE_LOG_LEVEL", cast=str, default="INFO"
    ).upper()
    TRANSACTION_MIDDLEWARE_LOG_FORMAT: str = config(
        "TRANSACTION_MIDDLEWARE_LOG_FORMAT",
        cast=str,
        default="<green>{time:YYYY-MM-DD HH:mm:ss}</green> | <level>{level: <8}</level> | <cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - <level>{message}</level>",
    )

    # Disable transaction for the whole application
    TRANSACTION_MIDDLEWARE_DISABLED = config(
        "TRANSACTION_MIDDLEWARE_DISABLED", cast=bool, default=False
    )


settings = Settings()



from typing import Optional

from fastapi import Request, status
from fastapi.security.utils import get_authorization_scheme_param
from ksuid import Ksuid
from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
from starlette.responses import JSONResponse, Response
from starlette.types import ASGIApp

from transaction_middleware.logging import logger


class TransactionMiddleware(BaseHTTPMiddleware):
    """Transaction middleware for FastAPI
    Manages transaction IDs in the request.

    Args:
        BaseHTTPMiddleware (_type_): _description_
    """

    def __init__(
        self,
        header_name: str = "X-Transaction-ID",
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.header_name = header_name

    async def dispatch(
        self,
        request: Request,
        call_next: RequestResponseEndpoint,
    ) -> Response | JSONResponse:

        # Check if transaction ID is already in the request
        transaction_id = request.headers.get(self.header_name)

        logger.debug("Transaction ID from request: {}", transaction_id)

        # Process the request
        response = await call_next(request)

        transaction_id = (
            request.state.transaction_id
            if hasattr(request.state, "transaction_id")
            else None
        )

        logger.debug("Transaction ID after request: {}", transaction_id)

        # Add transaction ID to the response
        if transaction_id:
            logger.debug(f"Adding Transaction ID to response: {transaction_id}")
            response.headers[self.header_name] = transaction_id
        return response



import sys

from loguru import logger

from transaction_middleware.settings import settings

# Configurar el logger
logger.remove()
logger.add(
    sink=sys.stderr,
    level=settings.TRANSACTION_MIDDLEWARE_LOG_LEVEL,
    format=settings.TRANSACTION_MIDDLEWARE_LOG_FORMAT,
    colorize=True,
)



from functools import wraps
from typing import List

from fastapi import HTTPException, Request
from ksuid import Ksuid

from transaction_middleware.logging import logger


def get_transaction_id(
    create_if_missing: bool = True,
    header_name: str = "X-Transaction-ID",
) -> str:
    """Get the transaction ID from the request headers or generate a new one.

    Args:
        create_if_missing (bool, optional): If True, generates a new transaction ID if not found in the request headers. Defaults to True.
        request (Request): _description_

    Raises:
        HTTPException: _description_

    Returns:
        str: _description_
    """

    def _get_transaction_id(request: Request) -> str:

        # First check if it exists on request.state
        transaction_id: str = (
            request.state.transaction_id
            if hasattr(request.state, "transaction_id")
            else None
        )

        logger.debug(f"Transaction ID from state: {transaction_id}")

        if not transaction_id:
            # Extract transaction id from request if does not exists on state
            transaction_id = request.headers.get(header_name)
            logger.debug(f"Transaction ID from headers: {transaction_id}")

        if transaction_id is None and create_if_missing:
            transaction_id = str(Ksuid())
            request.state.transaction_id = transaction_id

        return transaction_id

    return _get_transaction_id
